import TransactionService from './transaction.service'

export default {
  getOverview(config) {
    const ta = [
      { name: 'nd.bcd.list', old: { is_own: true, sorting_params_list: ['name'] } },
      { name: 'nd.bcd2ou.list', join: { 0: 'default' } },
      { name: 'org.unit.list', join: { 1: 'default' } },
      { name: 'nd.ip_subnet.list', join: { 0: 'default' } },
      { name: 'nd.vlan.list', join: { 0: 'default' } }
    ]
    return TransactionService.execute(config, ta)
  },
  getOverviewV4Only(config) {
    const ta = [
      { name: 'nd.ip_subnet.list', old: { is_own: true, type: '4' } },
      { name: 'nd.bcd.list', join: { 0: 'default' } },
      { name: 'nd.bcd2ou.list', join: { 1: 'default' } },
      { name: 'org.unit.list', join: { 2: 'default' } },
      { name: 'nd.vlan.list', join: { 1: 'default' } }
    ]
    return TransactionService.execute(config, ta)
  },
  getDetail(config, name) {
    const ta = [
      { name: 'nd.bcd.list', old: { name_list: [name] } },
      { name: 'nd.bcd2ou.list', join: { 0: 'default' } },
      { name: 'org.unit.list', join: { 1: 'default' }, old: { do_rootpath: true } },
      {
        name: 'nd.ip_subnet.list',
        join: { 0: 'default' },
        old: { sorting_params_list: ['type ASC', 'cidr ASC'] }
      },
      {
        name: 'dns.ip_addr.list',
        old: { is_reserved: true, sorting_params_list: ['value'] },
        join: { 3: 'api_fkey_dns_ip_addr_subnet' }
      },
      { name: 'nd.bcd2group.list', join: { 0: 'default' } },
      { name: 'cntl.group.list', join: { 5: 'default' } },
      { name: 'cntl.mgr2group.list', join: { 6: 'default' } },
      { name: 'cntl.mgr.list', join: { 7: 'default' } },
      { name: 'dns.fqdn2group.list', join: { 6: 'default' } },
      { name: 'cntl.mgr2ou.list', join: { 2: 'default' } },
      { name: 'cntl.mgr.list', join: { 10: 'default' } },
      { name: 'nd.vlan.list', join: { 0: 'default' } }
    ]
    return TransactionService.execute(config, ta)
  },
  getRecords(config, bcd) {
    const ta = [
      {
        name: 'dns.record.list',
        old: {
          sorting_params_list: ['type', 'target_ipaddr', 'data', 'fqdn'],
          target_bcd_list: [bcd],
          is_auth: true,
          target_bcd_list_is_nullable: true
        }
      },
      { name: 'dns.fqdn.list', join: { 0: 'default' } },
      { name: 'dnscfg.record_inttype.list', old: { is_own: true, sorting_params_list: ['record_type'] } },
      { name: 'dnscfg.record_type.list' },
      { name: 'dnscfg.fqdn_type.list', old: { sorting_params_list: ['position'] } },
      { name: 'dns.ip_addr.list', old: { type: '4', is_dhcp: true }, join: { 0: 'default' } }
    ]
    return TransactionService.execute(config, ta)
  }
}
