import TransactionService from './transaction.service'

export default {
  getByParent(config, parent) {
    const ta = [
      { name: 'dns.fqdn.list', old: { value: parent } },
      { name: 'dns.fqdn.list', join: { 0: 'api_fkey_dns_fqdn_parent' } }
    ]
    return TransactionService.execute(config, ta)
  },
  getRecrodsByTargetFQDN(config, fqdn) {
    const ta = [
      { name: 'dns.fqdn.list', old: { value: fqdn } },
      { name: 'dns.record.list', join: { 0: 'api_fkey_dns_record_target_fqdn' } },
      { name: 'dnscfg.fqdn_type.list', old: { sorting_params_list: ['position'] } },
      { name: 'dnscfg.record_inttype.list', old: { is_own: true, sorting_params_list: ['record_type'] } }
    ]
    return TransactionService.execute(config, ta)
  },
  getRecordsByFQDN(config, fqdn) {
    const ta = [
      { name: 'dns.fqdn.list', old: { value: fqdn } },
      { name: 'dns.record.list', old: { sorting_params_list: ['type', 'fqdn'] }, join: { 0: 'default' } },
      { name: 'dnscfg.fqdn_type.list', old: { sorting_params_list: ['position'] } },
      { name: 'dnscfg.record_inttype.list', old: { is_own: true, sorting_params_list: ['record_type'] } }
    ]
    return TransactionService.execute(config, ta)
  },
  getRecordsByIP(config, ip) {
    const ta = [
      { name: 'dns.ip_addr.list', old: { value_cidr: ip } },
      { name: 'dns.record.list', old: { sorting_params_list: ['type', 'fqdn'] }, join: {0: 'default'}},
      { name: 'dnscfg.fqdn_type.list', old: { sorting_params_list: ['position'] } },
      { name: 'dns.fqdn.list', join: { 1: 'default' } },
      { name: 'dnscfg.record_inttype.list', old: { is_own: true, sorting_params_list: ['record_type'] } }
    ]
    return TransactionService.execute(config, ta)
  },
  getOwnFQDNs(config) {
    const ta = [
      { name: 'cntl.mgr2ou.list', old: { is_own: true } },
      { name: 'cntl.mgr2group.list', old: { is_own: true } },
      { name: 'org.unit.list', join: { 0: 'default' } },
      { name: 'cntl.group.list', join: { 1: 'default' } },
      { name: 'dns.fqdn2ou.list', join: { 2: 'default'} },
      { name: 'dns.fqdn2group.list', join: { 3: 'default'} },
      { name: 'dns.fqdn.list', join: { 4: 'default' } },
      { name: 'dns.fqdn.list', join: { 5: 'default' } }
    ]
    return TransactionService.execute(config, ta)
  }
}
