import TransactionService from './transaction.service'
import apiutil from '@/util/apiutil'
import ipaddress from 'ipaddr.js'

function checkRegex(term) {
  if (term.startsWith('s/')) {
    return term.slice(2)
  }
  return '(?i).*' + term + '.*'
}

export default {
  searchBCDRegex(config, term, own) {
    const ta = [
      {
        name: 'nd.bcd.list',
        old: apiutil.denullify_dict({
          name_regexp: checkRegex(term),
          is_own: own,
          fetch_limit: 20,
          sorting_params_list: ['name']
        })
      }
    ]
    return TransactionService.execute(config, ta)
  },
  searchOURegex(config, term, own) {
    const ta = [
      {
        name: 'org.unit.list',
        old: apiutil.denullify_dict({
          name_regexp: checkRegex(term),
          fetch_limit: 20,
          sorting_params_list: ['short_name']
        })
      }
    ]
    return TransactionService.execute(config, ta)
  },
  searchOUDirect(config, term) {
    const ta = [
      {
        name: 'org.unit.list',
        old: apiutil.denullify_dict({
          short_name: term.toUpperCase(),
          fetch_limit: 20,
          sorting_params_list: ['short_name']
        })
      }
    ]
    return TransactionService.execute(config, ta)
  },
  searchGroupDirect(config, term) {
    const ta = [
      {
        name: 'cntl.group.list',
        old: apiutil.denullify_dict({
          name: term,
          fetch_limit: 20,
          sorting_params_list: ['name']
        })
      }
    ]
    return TransactionService.execute(config, ta)
  },
  searchFQDNRegex(config, term, own) {
    const ta = [
      {
        name: 'dns.fqdn.list',
        old: apiutil.denullify_dict({
          value_regexp: checkRegex(term),
          is_own: own,
          fetch_limit: 20,
          sorting_params_list: ['value']
        })
      }
    ]
    return TransactionService.execute(config, ta)
  },
  searchRecordRegex(config, term, own) {
    const ta = [
      {
        name: 'dns.record.list',
        old: apiutil.denullify_dict({
          fqdn_regexp: checkRegex(term),
          is_own: own,
          fetch_limit: 20,
          sorting_params_list: ['fqdn']
        })
      }
    ]
    return TransactionService.execute(config, ta)
  },
  searchRRDataRegex(config, term, own) {
    const ta = [
      {
        name: 'dns.record.list',
        old: apiutil.denullify_dict({
          target_data_unref_regexp: checkRegex(term),
          is_own: own,
          fetch_limit: 20,
          sorting_params_list: ['target_data_unref']
        })
      }
    ]
    return TransactionService.execute(config, ta)
  },
  searchRRByIP(config, ip, own) {
    const ta = [
      {
        name: 'dns.record.list',
        old: apiutil.denullify_dict({
          target_ipaddr_cidr: ip,
          is_own: own,
          fetch_limit: 20,
          sorting_params_list: ['target_ipaddr']
        })
      }
    ]
    return TransactionService.execute(config, ta)
  },
  searchVlan(config, term) {
    const ta = [
      {
        name: 'nd.vlan.list',
        old: apiutil.denullify_dict({
          id: term,
          fetch_limit: 20
        })
      }
    ]
    return TransactionService.execute(config, ta)
  },
  searchBCDSubnet(config, term, own) {
    const ta = [
      {
        name: 'nd.ip_subnet.list',
        old: apiutil.denullify_dict({
          cidr: term,
          is_own: own,
          fetch_limit: 20,
          cidr_operator: 'smallest_cts'
        })
      }
    ]
    return TransactionService.execute(config, ta)
  },
  searchBCDRegexTA(term, own, limit) {
    return {
      name: 'nd.bcd.list',
      old: apiutil.denullify_dict({
        name_regexp: checkRegex(term),
        is_own: own,
        fetch_limit: limit,
        sorting_params_list: ['name']
      })
    }
  },
  searchOURegexTA(term, own, limit) {
    return {
      name: 'org.unit.list',
      old: apiutil.denullify_dict({
        name_regexp: checkRegex(term),
        fetch_limit: limit,
        sorting_params_list: ['short_name']
      })
    }
  },
  searchOUShortRegexTA(term, own, limit) {
    return {
      name: 'org.unit.list',
      old: apiutil.denullify_dict({
        short_name_regexp: checkRegex(term),
        fetch_limit: limit,
        sorting_params_list: ['short_name']
      })
    }
  },
  searchOUDirectTA(term, limit) {
    return {
      name: 'org.unit.list',
      old: apiutil.denullify_dict({
        short_name: term.toUpperCase(),
        fetch_limit: limit,
        sorting_params_list: ['short_name']
      })
    }
  },
  searchGroupDirectTA(term, limit) {
    return {
      name: 'cntl.group.list',
      old: apiutil.denullify_dict({
        name: term,
        fetch_limit: limit,
        sorting_params_list: ['name']
      })
    }
  },
  searchFQDNRegexTA(term, own, limit) {
    return {
      name: 'dns.fqdn.list',
      old: apiutil.denullify_dict({
        value_regexp: checkRegex(term),
        is_own: own,
        fetch_limit: limit,
        sorting_params_list: ['value']
      })
    }
  },
  searchRecordRegexTA(term, own, limit) {
    return {
      name: 'dns.record.list',
      old: apiutil.denullify_dict({
        fqdn_regexp: checkRegex(term),
        is_own: own,
        fetch_limit: limit,
        sorting_params_list: ['fqdn']
      })
    }
  },
  searchRRDataRegexTA(term, own, limit) {
    return {
      name: 'dns.record.list',
      old: apiutil.denullify_dict({
        target_data_unref_regexp: checkRegex(term),
        is_own: own,
        fetch_limit: limit,
        sorting_params_list: ['target_data_unref']
      })
    }
  },
  searchRRByIPTA(ip, own, limit) {
    return {
      name: 'dns.record.list',
      old: apiutil.denullify_dict({
        is_own: own,
        fetch_limit: limit,
        sorting_params_list: ['target_ipaddr']
      }),
      join: { 0: 'default' }
    }
  },
  searchVlanTA(term, limit) {
    return {
      name: 'nd.vlan.list',
      old: apiutil.denullify_dict({
        id: term,
        fetch_limit: limit
      })
    }
  },
  searchBCDSubnetTA(term, own, limit) {
    return {
      name: 'nd.ip_subnet.list',
      old: apiutil.denullify_dict({
        cidr: term,
        is_own: own,
        fetch_limit: limit,
        cidr_operator: 'smallest_cts'
      })
    }
  },
  getIPsTA(ip, limit) {
    return {
      name: 'dns.ip_addr.list',
      old: {
        value_cidr: ip,
        fetch_limit: limit
      }
    }
  },
  searchGlobal(config, term, discriminator = null, limit = 20, own = true) {
    if (term === '') {
      return
    }
    const termIsNumeric = /^(0|[1-9][0-9]*)$/.test(term)
    const termIsIP = ipaddress.isValid(term.split('/')[0]) && !termIsNumeric
    const transactions = []
    const types = []
    if (termIsIP) {
      transactions.push(this.getIPsTA(term, limit))
      types.push(null)
    }
    const searches = {
      dns: [
        () => {
          if (termIsNumeric) {
            transactions.push(this.searchVlanTA(parseInt(term, limit)))
            types.push('nd.vlan')
          } else if (termIsIP) {
            transactions.push(this.searchRRByIPTA(term, own, limit))
            types.push('dns.record')
            transactions.push(this.searchBCDSubnetTA(term, own, limit))
            types.push('nd.ip_subnet')
          } else {
            transactions.push(this.searchBCDRegexTA(term, own, limit))
            types.push('nd.bcd')
            transactions.push(this.searchRecordRegexTA(term, own, limit))
            types.push('dns.record')
            transactions.push(this.searchRRDataRegexTA(term, own, limit))
            types.push('dns.record')
            transactions.push(this.searchFQDNRegexTA(term, own, limit))
            types.push('dns.fqdn')
          }
        }
      ],
      org: [
        () => {
          transactions.push(this.searchOURegexTA(term, null, limit))
          types.push('org.unit')
          transactions.push(this.searchOUShortRegexTA(term, null, limit))
          types.push('org.unit')
          transactions.push(this.searchOUDirectTA(term, limit))
          types.push('org.unit')
        }
      ],
      cntl: [
        () => {
          transactions.push(this.searchGroupDirectTA(term, limit))
          types.push('cntl.group')
        }
      ]
    }
    const discriminatorAliases = {
      dnsvs: 'dns'
    }
    if (discriminator == null) {
      for (const d in searches) {
        for (const s of searches[d]) {
          s()
        }
      }
    } else if (discriminator in searches) {
      for (const s of searches[discriminator]) {
        s()
      }
    } else if (discriminator in discriminatorAliases) {
      for (const s of searches[discriminatorAliases[discriminator]]) {
        s()
      }
    }
    return {
      promise: TransactionService.execute(config, transactions),
      types: types,
      term: discriminator ? discriminator + ':' + term : term
    }
  }
}
