import Vue from 'vue'
import Router from 'vue-router'

Vue.use(Router)

export default new Router({
  mode: 'history',
  base: process.env.BASE_URL,
  routes: [
    {
      path: '/',
      name: 'Home',
      component: () => import('./views/Home.vue'),
      meta: {
        resolveName: function () {
          return 'Home'
        }
      }
    },
    {
      path: '*',
      name: 'Ungültige URL!',
      component: () => import('./views/PageNotFound.vue'),
      meta: {
        resolveName: function () {
          return 'Ungültige URL!'
        }
      }
    },
    {
      path: '/swagger',
      name: 'swagger-ui',
      component: () => import('./views/Swagger.vue'),
      meta: {
        resolveName: function () {
          return 'API-Browser'
        }
      }
    },
    {
      path: '/launch/:component',
      name: 'launchpad',
      component: () => import('./views/Launchpad.vue'),
      meta: {
        resolveName: function () {
          return 'Launchpad'
        }
      }
    },
    {
      path: '/launch/:component/:pathMatch(.*)*',
      name: 'launchpad_payload',
      component: () => import('./views/Launchpad.vue'),
      meta: {
        resolveName: function () {
          return 'Launchpad'
        }
      }
    },
    {
      path: '/org/ou',
      name: 'ous',
      component: () => import('./views/org/OU'),
      meta: {
        resolveName: function () {
          return 'Organisationseinheiten'
        }
      }
    },
    {
      path: '/org/ou/:ou',
      name: 'ou',
      component: () => import('./views/org/OU'),
      meta: {
        resolveName: function (query) {
          return query.ou.toUpperCase()
        },
        resolveParents: function (query) {
          // const res = OUService.list(Vue.$store.state.netdb_axios_config, {do_rootpath: true, short_name: query.ou})
          return ['/org/ou']
        }
      }
    },
    {
      path: '/org/type',
      name: 'ou_types',
      component: () => import('./views/org/OUTypes'),
      meta: {
        resolveName: function () {
          return 'Organisationseinheitstypen'
        }
      }
    },
    {
      path: '/cntl/groups',
      name: 'groups',
      component: () => import('./views/cntl/Groups'),
      meta: {
        resolveName: function () {
          return 'Gruppen'
        }
      }
    },
    {
      path: '/cntl/groups/:group',
      name: 'group',
      component: () => import('./views/cntl/Groups'),
      meta: {
        resolveName: function (query) {
          return query.group.toUpperCase()
        },
        resolveParents: function (query) {
          // const res = OUService.list(Vue.$store.state.netdb_axios_config, {do_rootpath: true, short_name: query.ou})
          return ['/cntl/groups']
        }
      }
    },
    {
      path: '/login',
      name: 'login',
      component: () => import('./views/Login.vue'),
      meta: {
        resolveName: function () {
          return 'Login'
        }
      }
    },
    {
      path: '/oic_login',
      name: 'login_oic',
      component: () => import('./views/OICLogin.vue'),
      meta: {
        resolveName: function () {
          return 'OIC Login'
        }
      }
    },
    {
      path: '/user/tokens',
      name: 'tokens',
      component: () => import('./views/netdb/Tokens.vue'),
      meta: {
        resolveName: function () {
          return 'Accounts & Tokens'
        }
      }
    },
    {
      path: '/macauth',
      name: 'macauth',
      component: () => import('./views/macauth/MACAuth.vue'),
      meta: {
        resolveName: function () {
          return 'MACAuth'
        }
      }
    },
    {
      path: '/macauth/bcds/:bcd',
      name: 'macauth_object',
      component: () => import('./views/macauth/MACAuth.vue'),
      meta: {
        resolveName: function (query) {
          return query.bcd
        },
        resolveParents: function (query) {
          return ['/macauth']
        }
      }
    },
    {
      path: '/dhcp-leases',
      name: 'dhcp_leases',
      component: () => import('./views/dhcp_leases/Overview.vue'),
      meta: {
        resolveName: function () {
          return 'DHCP-Leases'
        }
      }
    },
    {
      path: '/dhcp-leases/bcds/:bcd',
      component: () => import('./views/dhcp_leases/Leases.vue'),
      meta: {
        resolveName: function (query) {
          return query.bcd
        },
        resolveParents: function () {
          return ['/dhcp-leases/']
        }
      }
    },
    {
      path: '/netdoc',
      name: 'netdoc',
      component: () => import('./views/nd/Netdoc'),
      meta: {
        resolveName: function () {
          return 'Netdoc'
        }
      }
    },
    {
      path: '/netdoc/map',
      name: 'map',
      component: () => import('./views/nd/Map.vue'),
      meta: {
        resolveName: function () {
          return 'Campusplan'
        }
      }
    },
    {
      path: '/tools/macfinder',
      name: 'macfinder',
      component: () => import('./views/tools/MACfinder.vue'),
      meta: {
        resolveName: function () {
          return 'MACfinder'
        }
      }
    },
    {
      path: '/tools/oui_lookup',
      name: 'ouilookup',
      component: () => import('./views/tools/OUILookup'),
      meta: {
        resolveName: function () {
          return 'MAC-Vendor-Suche'
        }
      }
    },
    {
      path: '/tools/ap_list',
      name: 'ap_list',
      component: () => import('./views/tools/APList'),
      meta: {
        resolveName: function () {
          return 'AP-Liste'
        }
      }
    },
    {
      path: '/requests',
      name: 'requests',
      component: () => import('./views/requests/Requests'),
      meta: {
        resolveName: function () {
          return 'Anträge'
        }
      }
    },
    {
      path: '/dnsvs',
      component: () => import('./views/dnsvs/Main.vue'),
      meta: {
        resolveName: function () {
          return 'DNSVS'
        }
      },
      children: [
        {
          path: '', beforeEnter: (to, from, next) => next('/dnsvs/bcds')
        },
        {
          path: 'fqdns',
          name: 'fqdns',
          component: () => import('./views/dnsvs/Entry.vue'),
          meta: {
            resolveName: function () {
              return 'FQDNs'
            }
          }
        },
        {
          path: 'fqdns/:fqdn',
          name: 'DomainInspect',
          component: () => import('./views/dnsvs/DomainInspect.vue'),
          meta: {
            resolveName: function (query) {
              return (query.fqdn === '$') ? '.' : query.fqdn
            },
            resolveParents (query) {
              const domains = query.fqdn.split('.').reverse()
              const res = ['/dnsvs/fqdns']
              let buf = ''
              for (let i = 0; i < domains.length - 1; i++) {
                let rbuf = ''
                const f = domains[i]
                if (f !== '' && f !== '$') {
                  buf = f + '.' + buf
                  rbuf = buf
                } else {
                  rbuf = '$'
                }
                res.unshift({ text: (rbuf === '$') ? '.' : rbuf, to: '/dnsvs/fqdns/' + rbuf })
              }
              return res
            }
          }
        },
        {
          path: 'fqdns/:fqdn/records',
          name: 'fqdn_records',
          component: () => import('./views/dnsvs/FQDNRecords.vue'),
          meta: {
            resolveName: function () {
              return 'Records'
            },
            resolveParents: function (query) {
              return [`/dnsvs/fqdns/${query.fqdn}`]
            }
          }
        },
        {
          path: 'fqdns/:fqdn/references',
          name: 'fqdn_references',
          component: () => import('./views/dnsvs/FQDNReferences.vue'),
          meta: {
            resolveName: function () {
              return 'Referenzen auf FQDN'
            },
            resolveParents: function (query) {
              return [`/dnsvs/fqdns/${query.fqdn}`]
            }
          }
        },
        {
          path: 'bcds',
          name: 'bcds',
          component: () => import('./views/dnsvs/Entry.vue'),
          meta: {
            resolveName: function () {
              return 'BCDs'
            }
          }
        },
        {
          path: 'bcds/:name',
          component: () => import('./views/dnsvs/BCDRecords.vue'),
          meta: {
            resolveName: function (query) {
              return query.name
            },
            resolveParents: function () {
              return ['/dnsvs/bcds']
            }
          }
        },
        {
          path: 'ips/:ip/records',
          name: 'ip_records',
          component: () => import('./views/dnsvs/IPRecords.vue'),
          meta: {
            resolveName: function () {
              return 'Records auf IP'
            }
          }
        }
      ]
    },
    { path: '/dnsvs/ranges', redirect: '/dnsvs/bcds' }
  ]
})
